# Package Management Module Implementation Plan

## Overview

The package management module will:
1. Provide a factory called `PackHero` that detects the current platform
2. Implement platform-specific package managers for Ubuntu (apt) and macOS (brew)
3. Support operations: install, remove, update, upgrade, list installed packages, search for packages, and check if a package is installed
4. Include debug functionality similar to buildah
5. Ensure all operations are non-interactive and have proper error propagation
6. Be wrapped in Rhai for scripting access

## Architecture

```mermaid
classDiagram
    class PackageError {
        +CommandFailed(String)
        +CommandExecutionFailed(std::io::Error)
        +UnsupportedPlatform(String)
        +Other(String)
    }

    class PackHero {
        -platform: Platform
        -debug: bool
        +new() PackHero
        +detect_platform() Platform
        +set_debug(bool) PackHero
        +debug() bool
        +install(package: &str) Result
        +remove(package: &str) Result
        +update() Result
        +upgrade() Result
        +list_installed() Result
        +search(query: &str) Result
        +is_installed(package: &str) Result
    }

    class Platform {
        <<enumeration>>
        Ubuntu
        MacOS
        Unknown
    }

    class PackageManager {
        <<interface>>
        +install(package: &str) Result
        +remove(package: &str) Result
        +update() Result
        +upgrade() Result
        +list_installed() Result
        +search(query: &str) Result
        +is_installed(package: &str) Result
    }

    class AptPackageManager {
        -debug: bool
        +new(debug: bool) AptPackageManager
        +install(package: &str) Result
        +remove(package: &str) Result
        +update() Result
        +upgrade() Result
        +list_installed() Result
        +search(query: &str) Result
        +is_installed(package: &str) Result
    }

    class BrewPackageManager {
        -debug: bool
        +new(debug: bool) BrewPackageManager
        +install(package: &str) Result
        +remove(package: &str) Result
        +update() Result
        +upgrade() Result
        +list_installed() Result
        +search(query: &str) Result
        +is_installed(package: &str) Result
    }

    PackHero --> Platform : uses
    PackHero --> PackageManager : uses
    PackageManager <|.. AptPackageManager : implements
    PackageManager <|.. BrewPackageManager : implements
```

## Implementation Details

### 1. Package Error Type

Create a custom error type for package management operations:

```rust
pub enum PackageError {
    CommandFailed(String),
    CommandExecutionFailed(std::io::Error),
    UnsupportedPlatform(String),
    Other(String),
}
```

### 2. Platform Detection

Implement platform detection to determine which package manager to use:

```rust
pub enum Platform {
    Ubuntu,
    MacOS,
    Unknown,
}

impl Platform {
    pub fn detect() -> Self {
        // Check for macOS
        if std::path::Path::new("/usr/bin/sw_vers").exists() {
            return Platform::MacOS;
        }
        
        // Check for Ubuntu
        if std::path::Path::new("/etc/lsb-release").exists() {
            // Read the file to confirm it's Ubuntu
            if let Ok(content) = std::fs::read_to_string("/etc/lsb-release") {
                if content.contains("Ubuntu") {
                    return Platform::Ubuntu;
                }
            }
        }
        
        Platform::Unknown
    }
}
```

### 3. Package Manager Trait

Define a trait for package managers to implement:

```rust
pub trait PackageManager {
    fn install(&self, package: &str) -> Result<CommandResult, PackageError>;
    fn remove(&self, package: &str) -> Result<CommandResult, PackageError>;
    fn update(&self) -> Result<CommandResult, PackageError>;
    fn upgrade(&self) -> Result<CommandResult, PackageError>;
    fn list_installed(&self) -> Result<Vec<String>, PackageError>;
    fn search(&self, query: &str) -> Result<Vec<String>, PackageError>;
    fn is_installed(&self, package: &str) -> Result<bool, PackageError>;
}
```

### 4. Platform-Specific Implementations

#### Ubuntu (apt) Implementation

```rust
pub struct AptPackageManager {
    debug: bool,
}

impl AptPackageManager {
    pub fn new(debug: bool) -> Self {
        Self { debug }
    }
}

impl PackageManager for AptPackageManager {
    fn install(&self, package: &str) -> Result<CommandResult, PackageError> {
        execute_package_command(&["apt-get", "install", "-y", package], self.debug)
    }
    
    fn remove(&self, package: &str) -> Result<CommandResult, PackageError> {
        execute_package_command(&["apt-get", "remove", "-y", package], self.debug)
    }
    
    fn update(&self) -> Result<CommandResult, PackageError> {
        execute_package_command(&["apt-get", "update", "-y"], self.debug)
    }
    
    fn upgrade(&self) -> Result<CommandResult, PackageError> {
        execute_package_command(&["apt-get", "upgrade", "-y"], self.debug)
    }
    
    fn list_installed(&self) -> Result<Vec<String>, PackageError> {
        let result = execute_package_command(&["dpkg", "--get-selections"], self.debug)?;
        let packages = result.stdout
            .lines()
            .filter_map(|line| {
                let parts: Vec<&str> = line.split_whitespace().collect();
                if parts.len() >= 2 && parts[1] == "install" {
                    Some(parts[0].to_string())
                } else {
                    None
                }
            })
            .collect();
        Ok(packages)
    }
    
    fn search(&self, query: &str) -> Result<Vec<String>, PackageError> {
        let result = execute_package_command(&["apt-cache", "search", query], self.debug)?;
        let packages = result.stdout
            .lines()
            .map(|line| {
                let parts: Vec<&str> = line.split_whitespace().collect();
                if !parts.is_empty() {
                    parts[0].to_string()
                } else {
                    String::new()
                }
            })
            .filter(|s| !s.is_empty())
            .collect();
        Ok(packages)
    }
    
    fn is_installed(&self, package: &str) -> Result<bool, PackageError> {
        let result = execute_package_command(&["dpkg", "-s", package], self.debug);
        match result {
            Ok(cmd_result) => Ok(cmd_result.success),
            Err(_) => Ok(false),
        }
    }
}
```

#### macOS (brew) Implementation

```rust
pub struct BrewPackageManager {
    debug: bool,
}

impl BrewPackageManager {
    pub fn new(debug: bool) -> Self {
        Self { debug }
    }
}

impl PackageManager for BrewPackageManager {
    fn install(&self, package: &str) -> Result<CommandResult, PackageError> {
        execute_package_command(&["brew", "install", package], self.debug)
    }
    
    fn remove(&self, package: &str) -> Result<CommandResult, PackageError> {
        execute_package_command(&["brew", "uninstall", package], self.debug)
    }
    
    fn update(&self) -> Result<CommandResult, PackageError> {
        execute_package_command(&["brew", "update"], self.debug)
    }
    
    fn upgrade(&self) -> Result<CommandResult, PackageError> {
        execute_package_command(&["brew", "upgrade"], self.debug)
    }
    
    fn list_installed(&self) -> Result<Vec<String>, PackageError> {
        let result = execute_package_command(&["brew", "list", "--formula"], self.debug)?;
        let packages = result.stdout
            .lines()
            .map(|line| line.trim().to_string())
            .filter(|s| !s.is_empty())
            .collect();
        Ok(packages)
    }
    
    fn search(&self, query: &str) -> Result<Vec<String>, PackageError> {
        let result = execute_package_command(&["brew", "search", query], self.debug)?;
        let packages = result.stdout
            .lines()
            .map(|line| line.trim().to_string())
            .filter(|s| !s.is_empty())
            .collect();
        Ok(packages)
    }
    
    fn is_installed(&self, package: &str) -> Result<bool, PackageError> {
        let result = execute_package_command(&["brew", "list", package], self.debug);
        match result {
            Ok(cmd_result) => Ok(cmd_result.success),
            Err(_) => Ok(false),
        }
    }
}
```

### 5. Command Execution with Debug Support

Implement a function to execute package management commands with debug support:

```rust
// Thread-local storage for debug flag
thread_local! {
    static DEBUG: std::cell::RefCell<bool> = std::cell::RefCell::new(false);
}

/// Set the debug flag for the current thread
pub fn set_thread_local_debug(debug: bool) {
    DEBUG.with(|cell| {
        *cell.borrow_mut() = debug;
    });
}

/// Get the debug flag for the current thread
pub fn thread_local_debug() -> bool {
    DEBUG.with(|cell| {
        *cell.borrow()
    })
}

/// Execute a package management command and return the result
pub fn execute_package_command(args: &[&str], debug: bool) -> Result<CommandResult, PackageError> {
    // Save the current debug flag
    let previous_debug = thread_local_debug();
    
    // Set the thread-local debug flag
    set_thread_local_debug(debug);
    
    if debug {
        println!("Executing command: {}", args.join(" "));
    }
    
    let output = Command::new(args[0])
        .args(&args[1..])
        .output();
    
    // Restore the previous debug flag
    set_thread_local_debug(previous_debug);
    
    match output {
        Ok(output) => {
            let stdout = String::from_utf8_lossy(&output.stdout).to_string();
            let stderr = String::from_utf8_lossy(&output.stderr).to_string();
            
            let result = CommandResult {
                stdout,
                stderr,
                success: output.status.success(),
                code: output.status.code().unwrap_or(-1),
            };
            
            // Always output stdout/stderr when debug is true
            if debug {
                if !result.stdout.is_empty() {
                    println!("Command stdout: {}", result.stdout);
                }
                
                if !result.stderr.is_empty() {
                    println!("Command stderr: {}", result.stderr);
                }
                
                if result.success {
                    println!("Command succeeded with code {}", result.code);
                } else {
                    println!("Command failed with code {}", result.code);
                }
            }
            
            if result.success {
                Ok(result)
            } else {
                // If command failed and debug is false, output stderr
                if !debug {
                    println!("Command failed with code {}: {}", result.code, result.stderr.trim());
                }
                Err(PackageError::CommandFailed(format!("Command failed with code {}: {}",
                    result.code, result.stderr.trim())))
            }
        },
        Err(e) => {
            // Always output error information
            println!("Command execution failed: {}", e);
            Err(PackageError::CommandExecutionFailed(e))
        }
    }
}
```

### 6. PackHero Factory

Implement the PackHero factory to provide a unified interface:

```rust
pub struct PackHero {
    platform: Platform,
    debug: bool,
}

impl PackHero {
    pub fn new() -> Self {
        let platform = Platform::detect();
        Self {
            platform,
            debug: false,
        }
    }
    
    pub fn set_debug(&mut self, debug: bool) -> &mut Self {
        self.debug = debug;
        self
    }
    
    pub fn debug(&self) -> bool {
        self.debug
    }
    
    fn get_package_manager(&self) -> Result<Box<dyn PackageManager>, PackageError> {
        match self.platform {
            Platform::Ubuntu => Ok(Box::new(AptPackageManager::new(self.debug))),
            Platform::MacOS => Ok(Box::new(BrewPackageManager::new(self.debug))),
            Platform::Unknown => Err(PackageError::UnsupportedPlatform("Unsupported platform".to_string())),
        }
    }
    
    pub fn install(&self, package: &str) -> Result<CommandResult, PackageError> {
        let pm = self.get_package_manager()?;
        pm.install(package)
    }
    
    pub fn remove(&self, package: &str) -> Result<CommandResult, PackageError> {
        let pm = self.get_package_manager()?;
        pm.remove(package)
    }
    
    pub fn update(&self) -> Result<CommandResult, PackageError> {
        let pm = self.get_package_manager()?;
        pm.update()
    }
    
    pub fn upgrade(&self) -> Result<CommandResult, PackageError> {
        let pm = self.get_package_manager()?;
        pm.upgrade()
    }
    
    pub fn list_installed(&self) -> Result<Vec<String>, PackageError> {
        let pm = self.get_package_manager()?;
        pm.list_installed()
    }
    
    pub fn search(&self, query: &str) -> Result<Vec<String>, PackageError> {
        let pm = self.get_package_manager()?;
        pm.search(query)
    }
    
    pub fn is_installed(&self, package: &str) -> Result<bool, PackageError> {
        let pm = self.get_package_manager()?;
        pm.is_installed(package)
    }
}
```

### 7. Rhai Integration

Update the Rhai OS module to include the package management functions:

```rust
// In rhai/os.rs

// Register package management functions
engine.register_fn("package_install", package_install);
engine.register_fn("package_remove", package_remove);
engine.register_fn("package_update", package_update);
engine.register_fn("package_upgrade", package_upgrade);
engine.register_fn("package_list", package_list);
engine.register_fn("package_search", package_search);
engine.register_fn("package_is_installed", package_is_installed);
engine.register_fn("package_set_debug", package_set_debug);

// Wrapper for os::package::install
pub fn package_install(package: &str) -> Result<String, Box<EvalAltResult>> {
    let hero = os::package::PackHero::new();
    hero.install(package)
        .map(|_| "Package installed successfully".to_string())
        .to_rhai_error()
}

// Wrapper for os::package::remove
pub fn package_remove(package: &str) -> Result<String, Box<EvalAltResult>> {
    let hero = os::package::PackHero::new();
    hero.remove(package)
        .map(|_| "Package removed successfully".to_string())
        .to_rhai_error()
}

// Wrapper for os::package::update
pub fn package_update() -> Result<String, Box<EvalAltResult>> {
    let hero = os::package::PackHero::new();
    hero.update()
        .map(|_| "Package lists updated successfully".to_string())
        .to_rhai_error()
}

// Wrapper for os::package::upgrade
pub fn package_upgrade() -> Result<String, Box<EvalAltResult>> {
    let hero = os::package::PackHero::new();
    hero.upgrade()
        .map(|_| "Packages upgraded successfully".to_string())
        .to_rhai_error()
}

// Wrapper for os::package::list_installed
pub fn package_list() -> Result<Array, Box<EvalAltResult>> {
    let hero = os::package::PackHero::new();
    let packages = hero.list_installed().to_rhai_error()?;
    
    // Convert Vec<String> to Rhai Array
    let mut array = Array::new();
    for package in packages {
        array.push(package.into());
    }
    
    Ok(array)
}

// Wrapper for os::package::search
pub fn package_search(query: &str) -> Result<Array, Box<EvalAltResult>> {
    let hero = os::package::PackHero::new();
    let packages = hero.search(query).to_rhai_error()?;
    
    // Convert Vec<String> to Rhai Array
    let mut array = Array::new();
    for package in packages {
        array.push(package.into());
    }
    
    Ok(array)
}

// Wrapper for os::package::is_installed
pub fn package_is_installed(package: &str) -> Result<bool, Box<EvalAltResult>> {
    let hero = os::package::PackHero::new();
    hero.is_installed(package).to_rhai_error()
}

// Global debug flag for package management
static mut PACKAGE_DEBUG: bool = false;

// Wrapper for setting package debug mode
pub fn package_set_debug(debug: bool) -> bool {
    unsafe {
        PACKAGE_DEBUG = debug;
        PACKAGE_DEBUG
    }
}
```

## Implementation Steps

1. Create the package.rs file in the os directory
2. Implement the PackageError enum
3. Implement the Platform enum with detection logic
4. Implement the PackageManager trait
5. Implement the AptPackageManager for Ubuntu
6. Implement the BrewPackageManager for macOS
7. Implement the debug functionality with thread-local storage
8. Implement the PackHero factory
9. Update os/mod.rs to include the package module
10. Update rhai/os.rs to include the package management functions
11. Test the implementation on both Ubuntu and macOS

## Testing Strategy

1. Create unit tests for each platform-specific implementation
2. Create integration tests that verify the PackHero factory correctly detects the platform and uses the appropriate package manager
3. Create Rhai examples that demonstrate the use of the package management functions